<xsl:stylesheet version="2.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output indent="yes" doctype-system="imdb-data.dtd" />

  <xsl:template match="/movies">
    <imdb-data>
      <xsl:apply-templates select="movie" />
      <!-- We first process each movie, copying it to the output
           excluding the cast. -->

      <xsl:for-each-group group-by="cast/actor" select="movie">
        <xsl:sort select="current-grouping-key()" />

        <!-- We then iterate over each distinct actor, in alphabetical
             order. -->
        
        <xsl:if test="count(current-group())>1">
          <!-- We only keep actors appearing in more than one movie. -->

          <actor name="{current-grouping-key()}">
            <xsl:for-each select="current-group()">
              <cast in="{generate-id()}"
                as="{cast/actor[.=current-grouping-key()]/@as}" />
              <!-- We show the movie identifier (generated by
                   generate-id() on the movie contextual node) and the
                   extracted character name. -->
            </xsl:for-each>
          </actor>
        </xsl:if>
      </xsl:for-each-group>
    </imdb-data>
  </xsl:template>

  <xsl:template match="movie">
    <!-- When processing a movie element, we copy it, add an
         automatically generated identifier attribute, and process its
         attribute (none here, but it does not hurt to plan for changes
         in the input structure) and child nodes. -->
    <xsl:copy>
      <xsl:attribute name="id" select="generate-id()" />
      <xsl:apply-templates select="@*|node()" />
    </xsl:copy>
  </xsl:template>

  <xsl:template match="cast" />
  <!-- The cast node is suppressed from the output. -->

  <xsl:template match="*">
    <!-- Redefinition of the default template for elements: instead of
         just browsing their children nodes, we also copy their structure
         in the output. -->
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" />
    </xsl:copy>
  </xsl:template>

  <xsl:template match="text()|@*"><xsl:copy /></xsl:template>
  <!-- Text and attribute nodes are copied as they are in the output. -->
</xsl:stylesheet>
